#include <Tiny4kOLED.h>
#include "RTClib.h"
#include <avr/sleep.h>
#include "PinChangeInterrupt.h"

int alarmPin = 1;
int buttonPin = 3;
int outputPin = 4;
const long twoHours = 7200000L;  // 2 hours delay = 7200000L (test was 30000)
const long halfHour = 1860000L;  // 30 minutes delay (actually 1 minute more to come out 30 minutes) = 1860000L; 65000 = little over a minute
volatile unsigned long previousMillis = 0;

RTC_DS3231 rtc;
volatile bool alarmInterrupt = false;

void setup() {
  pinMode(alarmPin, INPUT_PULLUP);
  pinMode(buttonPin, INPUT_PULLUP);
  pinMode(outputPin, OUTPUT);
  digitalWrite(outputPin, HIGH);

  if (!rtc.begin()) {
    abort();
  }

  rtc.disableAlarm(1);
  rtc.disableAlarm(2);
  rtc.clearAlarm(1);
  rtc.clearAlarm(2);
  rtc.writeSqwPinMode(DS3231_OFF);
  ADCSRA &= ~_BV(ADEN);

  oled.begin(128, 32, sizeof(tiny4koled_init_128x32), tiny4koled_init_128x32);
  clearDisplay();
  oled.on();
  oled.switchRenderFrame();
}

void loop() {
  prepareSleep();
}

void prepareSleep(){
  digitalWrite(outputPin, HIGH);  // LED comes on, also powers Display on
  rtc.setAlarm1(DateTime(2023, 11, 29, 5, 00, 00), DS3231_A1_Day);
  //rtc.setAlarm1(rtc.now() + TimeSpan(0, 0, 0, 20), DS3231_A1_Second);  // test
  char date[] = "DDD, DD MMM YYYY     Time  hh:mm:ss sleep";
  rtc.now().toString(date);
  oled.setRotation(1);
  oled.setFont(FONT6X8);
  oled.setCursor(6, 0);
  oled.print(date);
  oled.switchFrame();
  delay(5000);  // enough time to see current time
  clearDisplay();
  enterSleep();  // Go to sleep
}

void enterSleep() {
  oled.off();
  delay(500);                    // prevent re awakining
  digitalWrite(outputPin, LOW);  // turn off LED and Display
  sleep_enable();
  attachPCINT(digitalPinToPCINT(alarmPin), alarm_ISR, LOW);
  sei();
  set_sleep_mode(SLEEP_MODE_PWR_DOWN);
  sleep_mode();  // enters sleep

  // The program will continue from here when it wakes
  sleep_disable();  // Disable sleep mode
  digitalWrite(outputPin, HIGH);  // turn on LED and power to display. Clock is always powered on,
  // but resistor pack 1 is removed, clock LED resistor removed, and 4.7 k resistor added from SQW to Tiny's Vcc pin.
  // also remove coin cell charging diode or resistor to prevent swelling of standard CR2032 lithium battery.
  rtc.disableAlarm(1);  // Disable and clear alarm
  rtc.clearAlarm(1);

  oled.begin();
  clearDisplay();
  oled.on();
  oled.setFont(FONT6X8);
  oled.setCursor(6, 0);
  char date[] = "DDD, DD MMM YYYY     Time  hh:mm:ss woke";
  rtc.now().toString(date);
  oled.print(date);
  delay(2000);
  oled.setFont(FONT6X8);
  oled.setCursor(6, 3);
  oled.print(F(" Take Pill !!"));
  oled.switchFrame();
  previousMillis = millis();
  
  while ((millis() - previousMillis) <= twoHours) {  // 2 hr delay then go back to sleep
    int buttonState = digitalRead(buttonPin);
    if (buttonState == LOW) {  // Use a 1k resistor from button to interrupt pin for safety
      Display_Counter();       //break; break out if the button is pressed, otherwise wait for a 2-hour delay and if still no response, clear and go to sleep
    }
  }  // end waiting 2 hrs
  clearDisplay();
  prepareSleep();
}    

void Display_Counter() {
  // Display countdown timer
  clearDisplay();
  oled.setFont(FONT6X8);
  previousMillis = millis();
  int remainingTime = 32767;  // max int value initialize
  
  while ((millis() - previousMillis) <= halfHour) {
    oled.setCursor(18, 0);
    oled.print(F("Time Remaining:"));
    oled.setCursor(30, 2);
    remainingTime = (halfHour - (millis() - previousMillis)) / 60000;
    if(remainingTime < 10){
      oled.print(F(" "));
      oled.setCursor(36, 2);
    }
    oled.print(remainingTime);
    oled.setCursor(46, 2);
    oled.print(F("Minutes"));
    oled.switchFrame();
    int buttonState = digitalRead(buttonPin);
    if (buttonState == LOW || (remainingTime <= 0)) {
      clearDisplay();
      previousMillis = previousMillis + halfHour;  // break out of while loop in next step
      break;
    }
  }
  clearDisplay();
  prepareSleep();
}  // end of display counter

void clearDisplay() {
  oled.clear();
  oled.switchRenderFrame();  // clear all memory, frame 1
  oled.clear();
  oled.switchRenderFrame();  // clear all memory, frame 2
}
void alarm_ISR() {  // internal pullup on alarmPin
}
